export default class Ray {
    constructor(origin, direction) {
        this.origin = origin;
        this.direction = direction;
    }

    /**
     * Check if ray intersects with triangle using the Mueller-Trumbore intersection algorithm
     * https://en.wikipedia.org/wiki/M%C3%B6ller%E2%80%93Trumbore_intersection_algorithm
     * 
     * @param {*} triangle 
     */
    intersectsTriangle(triangle) {
        const v0v1 = triangle[1].sub(triangle[0]);
        const v0v2 = triangle[2].sub(triangle[0]);
        const pvec = this.direction.cross(v0v2);
        
        var det = v0v1.dot(pvec);
        
        if (det < 0.000001)
            return false;
        
        const invDet = 1.0 / det;
        const tvec = this.origin.sub(triangle[0]);
        const u = tvec.dot(pvec) * invDet;
        
        if (u < 0 || u > 1)
            return false;
        
        const qvec = tvec.cross(v0v1);
        const v = this.direction.dot(qvec) * invDet;
        
        if (v < 0 || u + v > 1)
            return false;
        
        return true;
    }
}