import GameObject from "./GameObject.js";
import Camera from "./Camera.js";
import { VRoomEngine } from "./VRoom.js";

export default class Scene {
    /**
     * 
     * @param {function} setupCallback Expected to return list of game objects
     */
    constructor(setupCallback=null) {
        this.gameObjects = [];
        this.setupCallback = setupCallback;
        this.initialized = false;
    }

    init(gl) {
        if (this.setupCallback) {
           this.gameObjects = this.setupCallback(this);
        }

        console.log("Scene init");
        if (!this.camera()) {
            console.warn("No camera found in the scene. Adding a default Camera at (0,0,0)");
            const cam = new GameObject();
            cam.addComponent(new Camera());
            this.gameObjects.push(cam);
        }

        this.gameObjects.forEach((go) => go.init(gl));
        this.initialized = true;
    }

    addObject(gameObject) {
        gameObject.init(VRoomEngine.gl);
        this.gameObjects.push(gameObject);
    }

    removeObject(gameObject) {
        const idx = this.gameObjects.indexOf(gameObject);
        
        if (idx >= 0)
            this.gameObjects.splice(idx, 1);
    }

    update(deltaTime) {
        this.gameObjects.forEach((go) => {
            if (go.update)
                go.update(deltaTime);
        });
    }

    render(renderInfo) {
        this.gameObjects.forEach((go) => {
            if (go.render) 
                go.render(renderInfo);
        });
    }

    /**
     * Get the first camera in the scene
     */
    camera() {
        return this.gameObjects.find((go) => go.hasComponent(Camera)) || null;
    }
}