import Vector3 from "./Vector3.js";
import Quaternion from "./Quaternion.js";

export default class Matrix4 {
    constructor(m00=0, m01=0, m02=0, m03=0, m10=0, m11=0, m12=0, m13=0, m20=0, m21=0, m22=0, m23=0, m30=0, m31=0, m32=0, m33=0) {
        this._mat = glMatrix.mat4.fromValues(m00, m01, m02, m03, m10, m11, m12, m13, m20, m21, m22, m23, m30, m31, m32, m33);
    }

    get glMat4() {
        return glMatrix.mat4.clone(this._mat);
    }

    clone() {
        return new Matrix4(this._mat[0], this._mat[1], this._mat[2], this._mat[3],
                           this._mat[4], this._mat[5], this._mat[6], this._mat[7],
                           this._mat[8], this._mat[9], this._mat[10], this._mat[11],
                           this._mat[12], this._mat[13], this._mat[14], this._mat[15]);
    }

    multiplyVec3(vec) {
        const result = new Vector3();
        glMatrix.vec3.transformMat4(result._vec, vec._vec, this._mat);
        return result;
    }

    multiplyMat4(mat) {
        const result = new Matrix4();
        glMatrix.mat4.multiply(result._mat, this._mat, mat._mat);
        return result;
    }

    multiply(matOrVec) {
        if (matOrVec instanceof Vector3) {
            return this.multiplyVec3(matOrVec);
        } else if (matOrVec instanceof Matrix4) {
            return this.multiplyMat4(matOrVec);
        }
    }

    inverse() {
        const mat = new Matrix4();
        glMatrix.mat4.invert(mat._mat, this._mat);
        return mat;
    }

    getRotation() {
        const quat = new Quaternion();
        glMatrix.mat4.getRotation(quat._quat, this._mat);
        return quat;
    }

    getTranslation() {
        const vec = new Vector3();
        glMatrix.mat4.getTranslation(vec._vec, this._mat);
        return vec;
    }

    static fromQuat(quat) {
        const mat = new Matrix4();
        glMatrix.mat4.fromQuat(mat._mat, quat.glQuat);
        return mat;
    }

    static fromEulerRotationTranslationScale(eulerRotation, translation, scale) {
        const quat = Quaternion.fromEuler(...eulerRotation.glVec3);
        return this.fromRotationTranslationScale(quat, translation, scale);
    }

    static fromRotationTranslationScale(rotation, translation, scale) {
        const mat = new Matrix4();
        glMatrix.mat4.fromRotationTranslationScale(mat._mat, rotation.glQuat, translation.glVec3, scale.glVec3);
        return mat;
    }
}