import Component from "../engine/Component.js";
import { VRoomEngine } from "../engine/VRoom.js";
import GameObject from "../engine/GameObject.js";
import Cube from "./Cube.js";
import CubeRenderer from "../engine/renderers/CubeRenderer.js";

const X_MIN = -5;
const X_MAX = 5;
const Y_MIN = 10;
const Y_MAX = 15;
const Z_MIN = -10;
const Z_MAX = -5;
const MIN_SPAWN_PERIOD = 0.2;
const MAX_SPAWN_PERIOD = 0.5;

function randomInt(min, max) {
    return Math.floor(Math.random() * (max - min)) + min;
}

function random(min, max) {
    return Math.random() * (max - min) + min;
}

export default class CubeSpawner extends Component {
    constructor() {
        super();

        this.currentTime = 0;
        this.nextSpawnTime = Math.random() * 0.5;
    }

    update(deltaTime) {
        this.currentTime += deltaTime;

        if (this.currentTime >= this.nextSpawnTime) {
            this.nextSpawnTime = this.currentTime + random(MIN_SPAWN_PERIOD, MAX_SPAWN_PERIOD);
            this.spawnedCubes++;
            this.spawnCube();
        }
    }

    spawnCube() {
        const cubeObject = new GameObject();
        cubeObject.layer = 1;
        cubeObject.addComponent(new Cube());
        cubeObject.addComponent(new CubeRenderer(1, [1, 1, 1, 1]));

        // set random position
        cubeObject.transform.setPosition(randomInt(X_MIN, X_MAX), randomInt(Y_MIN, Y_MAX), randomInt(Z_MIN, Z_MAX));
        cubeObject.transform.setRotation(randomInt(-180, 180), randomInt(-180, 180), randomInt(-180, 180));

        VRoomEngine.currentScene.addObject(cubeObject);
    }
}